package org.netkernel.module.standard;

import org.netkernel.layer0.meta.impl.SpaceMetaImpl;
import org.netkernel.module.standard.endpoint.IStandardEndpoint;
import org.netkernel.urii.IIdentifier;
import java.util.*;

public class StandardSpaceMeta extends SpaceMetaImpl
{
	private TreeMap<String,IStandardEndpoint> mMappingById;
	public static Comparator<String> sComparator;
	
	public StandardSpaceMeta(String aName, String aDescription, IIdentifier aIcon, IIdentifier aDoc)
	{	super(aName,aDescription,aIcon,aDoc);
		mMappingById = new TreeMap<String, IStandardEndpoint>(sComparator);
	}
	
	/** Only put if not present, first mapping wins
	 */
	public void putMapping(String aIdentifier, IStandardEndpoint aMapping)
	{	IStandardEndpoint removed=mMappingById.put(aIdentifier, aMapping);
		if (removed!=null)
		{	mMappingById.put(aIdentifier, removed);
		}
	}
	
	/** 
	 * @param aIdentifier
	 * @return
	 */
	public IStandardEndpoint getMappingWithId(String aIdentifier)
	{	return mMappingById.get(aIdentifier);
	}
	
	public String getAbsoluteIdentifier(String aIdentifier)
	{	String result=null;
		IStandardEndpoint mapping=mMappingById.get(aIdentifier);
		if (mapping!=null)
		{	result=aIdentifier;
		}
		else
		{	SortedMap<String,IStandardEndpoint> m=mMappingById.tailMap(aIdentifier);
			if (!m.isEmpty())
			{	String possibleMatch = m.firstKey();
				if (possibleMatch.endsWith(aIdentifier))
				{	result=possibleMatch;
				}
			}
		}
		return result;
	}
	
	static
	{	// reverse string compare
		sComparator = new Comparator<String>()
		{
			public int compare(String o1, String o2)
			{
				if (o1==o2) return 0;
				int i1=o1.length()-1;
				int i2=o2.length()-1;
				while(i1>=0 && i2>=0)
				{	int d = o1.charAt(i1)-o2.charAt(i2);
					if (d!=0)
					{	return d;
					}
					i1--;
					i2--;
				}
				return i1-i2;
			}
		};
	}
	
}
