package org.netkernel.module.standard.builtin;


import org.netkernel.container.IKernel;
import org.netkernel.layer0.bnf.BNFGrammarFactory;
import org.netkernel.layer0.bnf.IGrammarGroup;
import org.netkernel.layer0.meta.IIdentifierGrammar;
import org.netkernel.layer0.meta.impl.EndpointMetaBuilder;
import org.netkernel.layer0.nkf.INKFExpiryFunction;
import org.netkernel.layer0.nkf.INKFParameters;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.urii.ClassLoaderResourceExpiry;
import org.netkernel.layer0.urii.ClassLoaderResourceRepresentation;
import org.netkernel.layer0.urii.FileExpiry;
import org.netkernel.layer0.util.RegexRewriterRule;
import org.netkernel.layer0.util.Utils;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.endpoint.StandardMonoEndpointImpl;
import org.netkernel.request.IRequest;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.ISpace;
import org.w3c.dom.Element;

public class StaticResourceEndpoint extends StandardMonoEndpointImpl
{
	private RegexRewriterRule mRewrite;
	private String mToString;
	
	public StaticResourceEndpoint()
	{	declareThreadSafe();
		declareParameters(StaticResourcePrototype.sPrototypeParameters);
		declareUnhandledExceptionsExpired(false);
	}
	
	public void onCommissionEndpoint(IKernel aKernel, ISpace aSpace) throws Exception
	{	super.onCommissionEndpoint(aKernel, aSpace);		
		
		EndpointMetaBuilder b = new EndpointMetaBuilder();
		b.setName("Static Resource Endpoint");
		b.setVerbs(IRequest.VERB_SOURCE|IRequest.VERB_EXISTS);
		INKFParameters params = getParameters();
		
		String match=null;
		String glob = (String)params.getParamValue(StaticResourcePrototype.PARAM_GLOB);
		if(glob!=null)
		{	glob=glob.trim();
			if (glob.startsWith("/"))
			{	glob=glob.substring(1);
			}
			match=Utils.globStringToRegexString("res:/"+glob);
		}
		else
		{	String regex = (String)params.getParamValue(StaticResourcePrototype.PARAM_REGEX);
			if(regex!=null)
			{	match=regex.trim();
				String rewriteString = (String)params.getParamValue(StaticResourcePrototype.PARAM_REWRITE);
				if (rewriteString!=null)
				{	mRewrite = new RegexRewriterRule(match,rewriteString);
				}
			}
			else
			{	Element e=(Element)params.getParamValue(StaticResourcePrototype.PARAM_GRAMMAR);
				if(e!=null)
				{	e=XMLUtils.getChildElementNamed(e, "group");
					if(e!=null)
					{	IIdentifierGrammar grammar= BNFGrammarFactory.parse(e,aKernel.getLogger());
						b.setDescription("declaratively defined grammar");
						b.setGrammar(grammar);
					}
					else throw new Exception("grammar must contain a group");
				}
				else throw new Exception("fileset must provide one of: glob, regex or grammar");
			}
		}
		if(match!=null)
		{	IGrammarGroup root =  BNFGrammarFactory.createGroup(null,"uri",null);
			BNFGrammarFactory.createRegex(root, match);
			b.setDescription("matches "+match);
			mToString="Fileset "+match;
			b.setName(mToString);
			b.setGrammarRoot(root);
		}
		
		if (getParameters().getParamValue(PARAM_PRIVATE).equals(Boolean.TRUE))
		{	b.addField(PrivateFilterEndpoint.KEY_PRIVATE, Boolean.TRUE);
		}
		
		if (params.paramExists(StandardMonoEndpointImpl.PARAM_NAME))
		{	String s=(String)getParameters().getParamValue(StandardMonoEndpointImpl.PARAM_NAME);
			if  (s!=null)
			{	b.setName(s);
				mToString=s;
			}
		}
		if (params.paramExists(StandardMonoEndpointImpl.PARAM_DESCRIPTION))
		{	String s=(String)getParameters().getParamValue(StandardMonoEndpointImpl.PARAM_DESCRIPTION);
			if  (s!=null)
			{	b.setDescription(s);
			}
		}
		if (params.paramExists(StandardMonoEndpointImpl.PARAM_DOC))
		{	IIdentifier id=(IIdentifier)getParameters().getParamValue(StandardMonoEndpointImpl.PARAM_DOC);
			if  (id!=null)
			{	b.setHumanReadableDocumentation(id);
			}
		}
		if (params.paramExists(StandardMonoEndpointImpl.PARAM_ICON))
		{	IIdentifier id=(IIdentifier)getParameters().getParamValue(StandardMonoEndpointImpl.PARAM_ICON);
			if  (id!=null)
			{	b.setIcon(id);
			}
		}		
		
		
		
		
		mMeta=b.getMeta(aKernel.getLogger());
		
		String id = (String)params.getParamValue(StandardMonoEndpointImpl.PARAM_ID);
		if(id!=null)
		{	mId=id;
		}
		
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{	String path=getPathFromRequest(aContext);
		INKFResponse resp;
		ClassLoader classLoader=getSpace().getClassLoader();
		long pollPeriod = getKernel().getConfiguration().getLong("netkernel.poll",1000L);
		INKFExpiryFunction exp=null;
		Object rep=null;
		String mime=null;
		ClassLoaderResourceRepresentation representation= new ClassLoaderResourceRepresentation(path,classLoader);
		if (representation.getURL()!=null)
		{	mime=representation.getMime();
			rep=representation;
		}
		else
		{	rep = ClassLoaderResourceRepresentation.generateNotFoundException(getSpace(),path,aContext.getKernelContext().getKernel().getLogger());
		}
		 if (representation.getFile()!=null)
		{	exp= new FileExpiry(representation.getFile(),pollPeriod);
		}
		else
		{	exp= new ClassLoaderResourceExpiry(path,classLoader,pollPeriod);
		}
		resp=aContext.createResponseFrom(rep);
		resp.setExpiry(INKFResponse.EXPIRY_FUNCTION, exp);
		if(mime!=null)
		{	resp.setMimeType(mime);
		}
	}
	
	public void onExists(INKFRequestContext aContext) throws Exception
	{	String path=getPathFromRequest(aContext);
		ClassLoader classLoader=getSpace().getClassLoader();
		ClassLoaderResourceRepresentation representation= new ClassLoaderResourceRepresentation(path,classLoader);
		
		Boolean result = representation.getURL()==null?Boolean.FALSE:Boolean.TRUE;
		INKFResponse resp = aContext.createResponseFrom(result);
		
		long pollPeriod = getKernel().getConfiguration().getLong("netkernel.poll",1000L);
		if (representation.getFile()!=null)
		{	resp.setExpiry(INKFResponse.EXPIRY_FUNCTION, new FileExpiry(representation.getFile(),pollPeriod));
		}
		else
		{	resp.setExpiry(INKFResponse.EXPIRY_FUNCTION, new ClassLoaderResourceExpiry(path,classLoader,pollPeriod));
		}
	}
	
	private String getPathFromRequest(INKFRequestContext aContext)
	{	String path=aContext.getThisRequest().getIdentifier();
	
		if (mRewrite!=null)
		{	path=mRewrite.map(path);
		}
		int i=path.indexOf(':');
		path=path.substring(i+2);
		return path;
	}
	
	public String toString()
	{	return mToString;
	}
		
}
