/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  File:          $RCSfile: StandardAccessorImpl.java,v $
  Version:       $Name:  $ $Revision: 1.21 $
  Last Modified: $Date: 2008/06/11 14:06:03 $
 *****************************************************************************/

package org.netkernel.module.standard.endpoint;

import java.lang.reflect.Method;

import org.netkernel.container.IKernel;
import org.netkernel.layer0.bnf.*;
import org.netkernel.layer0.meta.*;
import org.netkernel.layer0.meta.impl.*;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.nkf.impl.NKFEndpointImpl;
import org.netkernel.layer0.util.PairList;
import org.netkernel.layer0.util.Utils;
import org.netkernel.module.standard.builtin.PrivateFilterEndpoint;
import org.netkernel.urii.*;

import org.w3c.dom.*;

/**
 * Base Class for Standard Module Accessors.
 * Uses declarative BNF grammar to implement resolution and metadata.
 * @author  pjr
 */
public abstract class StandardAccessorImpl extends StandardMonoEndpointImpl
{
	public static final String GRAMMAR_PARAM="grammar";
	private EndpointMetaBuilder mMetaBuilder;
	
	private static PairList sVerbMethods = new PairList(5);
	static
	{	sVerbMethods.put("onSource",new Integer(INKFRequestReadOnly.VERB_SOURCE));
		sVerbMethods.put("onSink",new Integer(INKFRequestReadOnly.VERB_SINK));
		sVerbMethods.put("onExists",new Integer(INKFRequestReadOnly.VERB_EXISTS));
		sVerbMethods.put("onDelete",new Integer(INKFRequestReadOnly.VERB_DELETE));
		sVerbMethods.put("onNew",new Integer(INKFRequestReadOnly.VERB_NEW));
		sVerbMethods.put("onRequest",new Integer(INKFRequestReadOnly.VERB_SOURCE|INKFRequestReadOnly.VERB_SINK|INKFRequestReadOnly.VERB_EXISTS|INKFRequestReadOnly.VERB_DELETE|INKFRequestReadOnly.VERB_NEW));
	}
	
	public static IPrototypeParameterMeta[] sPrototypeParametersLocal = new IPrototypeParameterMeta[]
    {	new PrototypeParameterMetaImpl(GRAMMAR_PARAM,Element.class,null)
   	};
	
	public static IPrototypeParameterMeta[] sPrototypeParameters =  mergeParameterArray(sPrototypeParametersLocal, StandardMonoEndpointImpl.sPrototypeParameters);
		
	/** Creates a new instance of StandardAccessorImpl */
	public StandardAccessorImpl()
	{	declareParameters(sPrototypeParameters);
		mMetaBuilder=new EndpointMetaBuilder();
		mMetaBuilder.setGrammarRoot(BNFGrammarFactory.NOTHING);
	}
	
	public void declareName(String aName)
	{	mMetaBuilder.setName(aName);
	}
	
	public void declareDescription(String aDescription)
	{	mMetaBuilder.setDescription(aDescription);
	}
	public void declareSourceRepresentation(Class aResponseClass)
	{	mMetaBuilder.addResponseClass(aResponseClass);
	}
	public void declareException(String aExceptionId, String aDescription)
	{	mMetaBuilder.addException(aExceptionId, aDescription);
	}
	public void declareSupportedVerbs(int aVerbs)
	{	mMetaBuilder.setVerbs(aVerbs);
	}
	public void declareGrammar(IGrammarGroup aGrammar)
	{	mMetaBuilder.setGrammarRoot(aGrammar);
	}
	public void declareGrammar(IIdentifierGrammar aGrammar)
	{	mMetaBuilder.setGrammar(aGrammar);
	}
	
	public void declareDocumentation(IIdentifier aDocumentationIdentifier)
	{	mMetaBuilder.setHumanReadableDocumentation(aDocumentationIdentifier);
	}
	
	public void declareIcon(IIdentifier aIconIdentifier)
	{	mMetaBuilder.setIcon(aIconIdentifier);
	}
	
	public void declareArgument(IEndpointArgumentMeta aArgumentMeta)
	{	mMetaBuilder.addArgument(aArgumentMeta);
	}
	
	public void onCommissionEndpoint(IKernel aKernel, ISpace aSpace) throws Exception
	{	
		super.onCommissionEndpoint(aKernel, aSpace);
		
		if (!mMetaBuilder.isNameSet())
		{	
			String id=null;
			if (getParameters().paramExists(PARAM_ID))
			{	id=(String)getParameters().getParamValue(PARAM_ID);
			}
			if(id!=null)
			{	mMetaBuilder.setName(mId);
			}
			else
			{	mMetaBuilder.setName(Utils.shortClassName(this.getClass()));
			}
		}
	
		if (getParameters().paramExists(GRAMMAR_PARAM))
		{	Element e=(Element)getParameters().getParamValue(GRAMMAR_PARAM);
			if  (e!=null)
			{	MetadataUtils.parseGrammar(e, mMetaBuilder, aKernel.getLogger());
			}
		}
		if (getParameters().paramExists(StandardMonoEndpointImpl.PARAM_NAME))
		{	String s=(String)getParameters().getParamValue(StandardMonoEndpointImpl.PARAM_NAME);
			if  (s!=null)
			{	declareName(s);
			}
		}
		if (getParameters().paramExists(StandardMonoEndpointImpl.PARAM_DESCRIPTION))
		{	String s=(String)getParameters().getParamValue(StandardMonoEndpointImpl.PARAM_DESCRIPTION);
			if  (s!=null)
			{	declareDescription(s);
			}
		}
		if (getParameters().paramExists(StandardMonoEndpointImpl.PARAM_DOC))
		{	IIdentifier id=(IIdentifier)getParameters().getParamValue(StandardMonoEndpointImpl.PARAM_DOC);
			if  (id!=null)
			{	declareDocumentation(id);
			}
		}
		if (getParameters().paramExists(StandardMonoEndpointImpl.PARAM_ICON))
		{	IIdentifier id=(IIdentifier)getParameters().getParamValue(StandardMonoEndpointImpl.PARAM_ICON);
			if  (id!=null)
			{	declareIcon(id);
			}
		}
		
		if (!mMetaBuilder.areVerbsSet())
		{	//test verbs
			int verbs=0;
			for (int i=sVerbMethods.size()-1; i>=0; i--)
			{	String methodName = (String)sVerbMethods.getValue1(i);
				Method m=getClass().getMethod(methodName, new Class[]{INKFRequestContext.class});
				if (m!=null && m.getDeclaringClass()!=NKFEndpointImpl.class)
				{	int verb = ((Integer)sVerbMethods.getValue2(i)).intValue();
					verbs|=verb;
				}
			}
			if (verbs==0 && !StandardTransportImpl.class.isAssignableFrom(this.getClass()))
			{	// this helps out developer when just created a clean accessor
				verbs=INKFRequestReadOnly.VERB_SOURCE;
			}
			mMetaBuilder.setVerbs(verbs);
		}
		
		if (getParameters().paramExists(PARAM_PRIVATE) && getParameters().getParamValue(PARAM_PRIVATE).equals(Boolean.TRUE))
		{	mMetaBuilder.addField(PrivateFilterEndpoint.KEY_PRIVATE, Boolean.TRUE);
		}
		
		mMeta=mMetaBuilder.getMeta(aKernel.getLogger());
		mMetaBuilder=null;
	}
	
}
