package org.netkernel.module.standard.endpoint;

import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Set;

import javax.xml.parsers.ParserConfigurationException;

import org.netkernel.container.IKernel;
import org.netkernel.container.ILogger;
import org.netkernel.container.IMessages;
import org.netkernel.layer0.meta.IEndpointStateMeta;
import org.netkernel.layer0.meta.IPhysicalEndpointFieldMeta;
import org.netkernel.layer0.meta.IPhysicalEndpointMeta;
import org.netkernel.layer0.meta.IPrototypeParameterMeta;
import org.netkernel.layer0.meta.ISpaceElements;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.nkf.impl.*;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.util.Utils;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.SpaceFactory;
import org.netkernel.module.standard.StandardSpace;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestResponseFieldsImpl;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.ISpace;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


public abstract class StandardEndpointImpl extends NKFEndpointImpl implements IStandardEndpoint, IEndpointStateMeta
{
	private boolean mPostCommissionFlag = false;
	private static final Class[] sPostCommissionParameters = new Class[] { INKFRequestContext.class };
	
	public StandardEndpointImpl()
	{	
		//test for existence of postCommission method that needs to be fired
		Class c=this.getClass();
		while(!mPostCommissionFlag && c!=StandardEndpointImpl.class)
		{	try
			{	Method m=c.getDeclaredMethod("postCommission", sPostCommissionParameters);
				mPostCommissionFlag = true;
			}
			catch (Exception e)	{ }
			try
			{	Method m=c.getDeclaredMethod("doPostCommission", sPostCommissionParameters);
				mPostCommissionFlag = true;
			}
			catch (Exception e)	{ }
			
			c=c.getSuperclass();
		}
		
		setParameters(Parameters.EMPTY);
	}
	
	public void declareParameters(IPrototypeParameterMeta[] aParameterDefn)
	{	INKFParametersControl parameters;
		if (aParameterDefn!=null)
		{	boolean tolerant = getParameters().isTolerant();
			parameters=new Parameters(aParameterDefn);
			parameters.setTolerant(tolerant);
		}
		else
		{	parameters = Parameters.EMPTY;
		}
		setParameters(parameters);
	}
	
	public void declareTolerantParameters()
	{	
		getParameters().setTolerant(true);
	}
	
	public static IPrototypeParameterMeta[] mergeParameterArray(IPrototypeParameterMeta[] aParameters1, IPrototypeParameterMeta[] aParameters2)
	{	IPrototypeParameterMeta[] result = new IPrototypeParameterMeta[aParameters1.length+aParameters2.length];
		Set s = new HashSet();
		int j=0;
		for (int i=0; i<aParameters1.length; i++)
		{	result[j++]=aParameters1[i];
			if (!s.add(aParameters1[i].getName()))
			{	throw new IllegalArgumentException("duplicate parameter ["+aParameters1[i].getName()+"]");
			}
		}
		for (int i=0; i<aParameters2.length; i++)
		{	result[j++]=aParameters2[i];
			if (!s.add(aParameters2[i].getName()))
			{	throw new IllegalArgumentException("duplicate parameter ["+aParameters2[i].getName()+"]");
			}
		}
		return result;
	}	
	
	public void setParameter(String aParameterName, Element aParameterElement, IKernel aKernel) throws NKFException
	{	getParameters().setParameter(aParameterName,aParameterElement,aKernel);
	}
	
	public void setParameterRaw(String aParameterName, Object aValue, IKernel aKernel) throws NKFException
	{	getParameters().setParameterRaw(aParameterName,aValue);
	}
	
	public void onCommissionEndpoint(IKernel aKernel, ISpace aSpace) throws Exception
	{	super.onCommissionEndpoint(aKernel, aSpace);
		
		if (getParameters()!=Parameters.EMPTY)
		{	getParameters().onCommission(aKernel,aSpace);
		}
	}
	
	public void postCommissionMapping(IRequestScopeLevel aStaticScope) throws Exception
	{	innerPostCommission(aStaticScope);
	}
	
	public void preDecommissionMapping(IRequestScopeLevel aStaticScope) throws Exception
	{	boolean hasDecommissionMethod=false;
	
		Class c=this.getClass();
		do
		{	try
			{	Method m=c.getDeclaredMethod("preDecommission", sPostCommissionParameters);
				hasDecommissionMethod = true;
			}
			catch (Exception e)	{ }
			
			c=c.getSuperclass();
		} while(!hasDecommissionMethod && c!=StandardEndpointImpl.class);
		
		if (hasDecommissionMethod)
		{	NKFLifecycleEventContextImpl context = new NKFLifecycleEventContextImpl(getKernel(),aStaticScope,this,getParameters());
			preDecommission(context);
			context.invalidate();
		}
	}
	
	public void onResolve(INKFResolutionContext aContext) throws Exception
	{	if (mPostCommissionFlag==true)
		{	innerPostCommission(aContext.getKernelContext().getThisKernelRequest().getRequestScope());
		}
	}
	
	protected final void innerPostCommission(IRequestScopeLevel aStaticScope) throws Exception
	{	if (mPostCommissionFlag==true)
		{	mPostCommissionFlag=false;
			try
			{	NKFLifecycleEventContextImpl context = new NKFLifecycleEventContextImpl(getKernel(),aStaticScope,this,getParameters());
				doPostCommission(context);
				context.invalidate();
			}
			catch (Throwable e)
			{	ILogger logger=getKernel().getLogger();
				String ex=Utils.throwableToString(e);
				String msg=logger.format("MSG_STD_ENDPOINT_POSTCOMMISSION_FAILED",this.toString(),ex);
				logger.logRaw(ILogger.WARNING,this,msg);
				setInitialisationProblem(msg);
			}
		}
	}
	
	/** A layer of indirection to allow subclasses to intercept and do work
	 */
	void doPostCommission(INKFRequestContext aContext) throws Exception
	{	postCommission(aContext);
	}
	
	/** Override this method to provide post commission initialisation
	 * @param aContext 
	 * @throws Exception throw what you like - don't try to handle exceptions unless you can do something useful
	 */
	protected void postCommission(INKFRequestContext aContext) throws Exception
	{
	}
	
	/** Override this method to provide pre decommission cleanup
	 * @param aContext
	 * @throws Exception throw what you like - don't try to handle exceptions unless you can do something useful
	 */
	protected void preDecommission(INKFRequestContext aContext) throws Exception
	{
	}
	
	public Object getParameter(String aName) throws NKFException
	{	return getParameters().getParamValue(aName);
	}
	
	protected StandardSpace getStandardSpace()
	{	return (StandardSpace)getSpace();
	}
	
	public IRequestResponseFields getState()
	{	return mProblems;
	}
	
	public IPhysicalEndpointFieldMeta[] getStateMeta()
	{	return IPhysicalEndpointMeta.NO_ADDITIONAL_FIELDS_META;
	}
	
	public void setState(String aKey, Object aValue) throws IllegalArgumentException
	{	throw new IllegalArgumentException();
	}
	
	public void setInitialisationProblem(String aProblem)
	{
		if (mProblems==RequestResponseFieldsImpl.EMPTY)
		{	mProblems = new RequestResponseFieldsImpl(10);
		}
		((RequestResponseFieldsImpl)mProblems).put(IEndpointStateMeta.KEY_PROBLEM, aProblem);
	}
	
	public void clearInitialisationProblems()
	{	mProblems=RequestResponseFieldsImpl.EMPTY;
	}
	
	private IRequestResponseFields mProblems = RequestResponseFieldsImpl.EMPTY;
		
	public String toString()
	{	return Utils.shortClassName(this.getClass());
	}

	public ISpace[] getSpaces(INKFRequestContext context) throws Exception
	{	return IStandardEndpoint.NO_SPACES;
	}
	
}